% This file loads the results of the simulations.
%
% Author: Antoine Falisse
% Date: 9/9/2019
%
%% load results
pathresults = [pathRepo_2D,'/Results/',setup.ocp];
load([pathresults,'/Results_2D_Paper.mat']);

%% Extract results following pre-defined structure
deri        = settings(ww_2D(k),1);	% derivative supplier identifier
hessi       = settings(ww_2D(k),2);	% Hessian identifier
linsoli     = settings(ww_2D(k),3);	% linear solver identifier
exp_E       = settings(ww_2D(k),4);	% power metabolic energy rate
IGi         = settings(ww_2D(k),5);	% initial guess identifier
N           = 50;                  	% number of mesh intervals
if deri == 1
    setup.derivatives = 'AD_Recorder'; % Algorithmic differentiation / Recorder     
elseif deri == 2
    setup.derivatives = 'AD_ADOLC'; % Algorithmic differentiation / ADOL-C
elseif deri == 3
    setup.derivatives = 'FD'; % Finite differences
end
hess_names = {'Approximated','Exact'};
linear_solvers = {'mumps','ma27','ma57','ma77','ma86','ma97'}; 
% Joints angles
Qs_opt_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).q_opt_GC;
% Joints velocities
Qdots_opt_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).qdot_opt_GC;
% Muscle activations
Acts_opt_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).a_opt_GC;
% Joint torques
Ts_opt_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).tau_opt_GC;
% Ground reaction forces
GRFs_opt_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).GRF_opt_GC;
% Stats from CasADi
Stats_2D(ww_2D(k)).m = ...
    Results_2D.(['Derivative_',setup.derivatives]). ...
    (['Hessian_',hess_names{hessi}]). ...
    (['LinearSolver_',linear_solvers{linsoli}]). ...    
    (['MetabolicEnergyRate_',num2str(exp_E)]). ...
    (['InitialGuess_',num2str(IGi)]).stats;
   